/***
*
*	File derived from bspfile.c, cmdlib.c and scriplib.c
*	File created by Jeroen "ShadowLord" Bogers (resgen@hltools.com)
*
*	Orriginal file copyrights:
*
*	Copyright (c) 1998, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
****/

#include "bspent.h"

// ===================================================================================================
// bspfile.c

int entdatasize;
char *dentdata;

eheader_t *eheader;

/*
=============
filelength
=============
*/
int filelength (FILE *f)
{
	int		pos;
	int		end;

	pos = ftell (f); // Save the position for reference
	fseek (f, 0, SEEK_END); // Set the file pointer to the end
	end = ftell (f); // Get the current pointer psoition (the end)
	fseek (f, pos, SEEK_SET); // Reset the pointer to it's orriginal place

	return end;
}


/*
=============
LoadEntHeader
=============
*/
int LoadEntHeader (FILE *f)
{
	int    length;
	void *buffer; // Temp buffer

	length = filelength (f); // Get the filelength
	if (length < sizeof(*eheader)) { return -1; } // 0 length - empty file/file too small

	buffer = malloc (sizeof(*eheader)); // Create a temp buffer for the header

	if (fread (buffer, 1, sizeof(*eheader), f) != sizeof(*eheader))
	{
		free (buffer); // free the buffer, we don't want it's mem to stay reserved on error!
		return -1; // The reading failed
	}

	eheader = (eheader_t *)buffer; // Move the buffer pointer into the header pointer

	return length; // return our file-length
}


/*
=============
LoadBSPEntityInfo
=============
*/
int LoadBSPEntityInfo (char *filename)
{
	int filelen; // The length of the file
	int length, ofs; // The length and offset of the entity data
	char *entdata; // The entity data

	FILE *f; // The file we read

	f = fopen(filename, "rb"); // Open the file for reading (binary reading)
	if (!f) { return 1; } // Did we open the file?

	filelen = LoadEntHeader(f);
	if (filelen < 0) { return 1; } // Header loading error

	if (eheader->version != BSPVERSION)
	{
		// Wrong BSP version
		free (eheader); // free the header, we don't want it's mem to stay reserved on error!
		return 1;
	}

	length = eheader->lumps[0].filelen; // Give us the position of the start of the entdata
	entdatasize = length; // MAke the entdatasize equal to the length

	ofs = eheader->lumps[0].fileofs; // Give us the length of the entdata

	free (eheader); // free the header, we don't need it anymore

	// Verify the validity of the data
	if (ofs + length >= filelen)
		return 2; // File corrupted

	if (ofs <= 0)
		return 2; // File corrupted

	if (length <= 0)
		return 2; // File corrupted
	
	
	entdata = new char[length+1]; // Create the array to store the entdata in.

	// Set the pointer of the file to the ofset, and read length	
	fseek (f, ofs, SEEK_SET); // Set the file pointer to the start of the entity data
	if ((int)fread (entdata, 1, length, f) != length)
	{
		return 1; // The reading failed
	}
	entdata[length] = 0; // Add terminating NULL

	dentdata = entdata; // Set dentdata to entdata

	// we have the data close the file and return
	fclose (f);

	return 0;
}

void UnLoadBSPEntityInfo()
{
	delete [] dentdata; // Free the dentdata mem
}

// ===================================================================================================
// File loading

FILE *SafeOpenRead (char *filename)
{
	FILE	*f;

	f = fopen(filename, "rb");

	//if (!f)
		//Error ("Error opening %s: %s",filename,strerror(errno));

	return f;
}


int SafeRead (FILE *f, void *buffer, int count)
{
	if ( fread (buffer, 1, count, f) != (size_t)count)
		return 1;//Error ("File read failure")

	return 0;
}


/*
==============
LoadFile
==============
*/
int    LoadFile (char *filename, void **bufferptr)
{
	FILE	*f;
	int    length;
	void    *buffer;

	f = SafeOpenRead (filename);
	if (!f) { return -1; }
	length = filelength (f);
	buffer = malloc (length+1);
	((char *)buffer)[length] = 0;
	if (SafeRead (f, buffer, length) != 0) { return -1; }
	fclose (f);

	*bufferptr = buffer;
	return length;
}
